@@ -46,6 +46,10 @@ const (
 	alertStateLabel = "alertstate"
 )
 
+// ErrDuplicateAlertLabelSet is returned when an alerting rule evaluation produces
+// metrics with identical labelsets after applying alert labels.
+var ErrDuplicateAlertLabelSet = errors.New("vector contains metrics with the same labelset after applying alert labels")
+
 // AlertState denotes the state of an active alert.
 type AlertState int
 
@@ -441,7 +445,7 @@ func (r *AlertingRule) Eval(ctx context.Context, queryOffset time.Duration, ts t
 		resultFPs[h] = struct{}{}
 
 		if _, ok := alerts[h]; ok {
-			return nil, errors.New("vector contains metrics with the same labelset after applying alert labels")
+			return nil, ErrDuplicateAlertLabelSet
 		}
 
 		alerts[h] = &Alert{
