@@ -215,7 +215,7 @@ func newScrapePool(cfg *config.ScrapeConfig, app storage.Appendable, offsetSeed
 			opts.alwaysScrapeClassicHist,
 			opts.convertClassicHistToNHCB,
 			cfg.ScrapeNativeHistogramsEnabled(),
-			options.EnableCreatedTimestampZeroIngestion,
+			options.EnableStartTimestampZeroIngestion,
 			options.EnableTypeAndUnitLabels,
 			options.ExtraMetrics,
 			options.AppendMetadata,
@@ -309,6 +309,7 @@ func (sp *scrapePool) stop() {
 		sp.metrics.targetScrapePoolTargetsAdded.DeleteLabelValues(sp.config.JobName)
 		sp.metrics.targetScrapePoolSymbolTableItems.DeleteLabelValues(sp.config.JobName)
 		sp.metrics.targetSyncIntervalLength.DeleteLabelValues(sp.config.JobName)
+		sp.metrics.targetSyncIntervalLengthHistogram.DeleteLabelValues(sp.config.JobName)
 		sp.metrics.targetSyncFailed.DeleteLabelValues(sp.config.JobName)
 	}
 }
@@ -505,6 +506,9 @@ func (sp *scrapePool) Sync(tgs []*targetgroup.Group) {
 	sp.metrics.targetSyncIntervalLength.WithLabelValues(sp.config.JobName).Observe(
 		time.Since(start).Seconds(),
 	)
+	sp.metrics.targetSyncIntervalLengthHistogram.WithLabelValues(sp.config.JobName).Observe(
+		time.Since(start).Seconds(),
+	)
 	sp.metrics.targetScrapePoolSyncsCounter.WithLabelValues(sp.config.JobName).Inc()
 }
 
@@ -712,13 +716,11 @@ func mutateSampleLabels(lset labels.Labels, target *Target, honor bool, rc []*re
 		}
 	}
 
-	res := lb.Labels()
-
-	if len(rc) > 0 {
-		res, _ = relabel.Process(res, rc...)
+	if keep := relabel.ProcessBuilder(lb, rc...); !keep {
+		return labels.EmptyLabels()
 	}
 
-	return res
+	return lb.Labels()
 }
 
 func resolveConflictingExposedLabels(lb *labels.Builder, conflictingExposedLabels []labels.Label) {
@@ -951,7 +953,7 @@ type scrapeLoop struct {
 
 	alwaysScrapeClassicHist  bool
 	convertClassicHistToNHCB bool
-	enableCTZeroIngestion    bool
+	enableSTZeroIngestion    bool
 	enableTypeAndUnitLabels  bool
 	fallbackScrapeProtocol   string
 
@@ -996,11 +998,9 @@ type scrapeCache struct {
 	// be a pointer so we can update it.
 	droppedSeries map[string]*uint64
 
-	// seriesCur and seriesPrev store the labels of series that were seen
-	// in the current and previous scrape.
-	// We hold two maps and swap them out to save allocations.
-	seriesCur  map[uint64]*cacheEntry
-	seriesPrev map[uint64]*cacheEntry
+	// Series that were seen in the current and previous scrape, for staleness detection.
+	seriesCur  map[storage.SeriesRef]*cacheEntry
+	seriesPrev map[storage.SeriesRef]*cacheEntry
 
 	// TODO(bwplotka): Consider moving Metadata API to use WAL instead of scrape loop to
 	// avoid locking (using metadata API can block scraping).
@@ -1027,8 +1027,8 @@ func newScrapeCache(metrics *scrapeMetrics) *scrapeCache {
 	return &scrapeCache{
 		series:        map[string]*cacheEntry{},
 		droppedSeries: map[string]*uint64{},
-		seriesCur:     map[uint64]*cacheEntry{},
-		seriesPrev:    map[uint64]*cacheEntry{},
+		seriesCur:     map[storage.SeriesRef]*cacheEntry{},
+		seriesPrev:    map[storage.SeriesRef]*cacheEntry{},
 		metadata:      map[string]*metaEntry{},
 		metrics:       metrics,
 	}
@@ -1076,13 +1076,9 @@ func (c *scrapeCache) iterDone(flushCache bool) {
 		c.metaMtx.Unlock()
 	}
 
-	// Swap current and previous series.
+	// Swap current and previous series then clear the new current, to save allocations.
 	c.seriesPrev, c.seriesCur = c.seriesCur, c.seriesPrev
-
-	// We have to delete every single key in the map.
-	for k := range c.seriesCur {
-		delete(c.seriesCur, k)
-	}
+	clear(c.seriesCur)
 
 	c.iter++
 }
@@ -1119,13 +1115,13 @@ func (c *scrapeCache) getDropped(met []byte) bool {
 	return ok
 }
 
-func (c *scrapeCache) trackStaleness(hash uint64, ce *cacheEntry) {
-	c.seriesCur[hash] = ce
+func (c *scrapeCache) trackStaleness(ref storage.SeriesRef, ce *cacheEntry) {
+	c.seriesCur[ref] = ce
 }
 
 func (c *scrapeCache) forEachStale(f func(storage.SeriesRef, labels.Labels) bool) {
-	for h, ce := range c.seriesPrev {
-		if _, ok := c.seriesCur[h]; !ok {
+	for ref, ce := range c.seriesPrev {
+		if _, ok := c.seriesCur[ref]; !ok {
 			if !f(ce.ref, ce.lset) {
 				break
 			}
@@ -1264,7 +1260,7 @@ func newScrapeLoop(ctx context.Context,
 	alwaysScrapeClassicHist bool,
 	convertClassicHistToNHCB bool,
 	enableNativeHistogramScraping bool,
-	enableCTZeroIngestion bool,
+	enableSTZeroIngestion bool,
 	enableTypeAndUnitLabels bool,
 	reportExtraMetrics bool,
 	appendMetadataToWAL bool,
@@ -1321,7 +1317,7 @@ func newScrapeLoop(ctx context.Context,
 		timeout:                       timeout,
 		alwaysScrapeClassicHist:       alwaysScrapeClassicHist,
 		convertClassicHistToNHCB:      convertClassicHistToNHCB,
-		enableCTZeroIngestion:         enableCTZeroIngestion,
+		enableSTZeroIngestion:         enableSTZeroIngestion,
 		enableTypeAndUnitLabels:       enableTypeAndUnitLabels,
 		fallbackScrapeProtocol:        fallbackScrapeProtocol,
 		enableNativeHistogramScraping: enableNativeHistogramScraping,
@@ -1426,6 +1422,9 @@ func (sl *scrapeLoop) scrapeAndReport(last, appendTime time.Time, errc chan<- er
 		sl.metrics.targetIntervalLength.WithLabelValues(sl.interval.String()).Observe(
 			time.Since(last).Seconds(),
 		)
+		sl.metrics.targetIntervalLengthHistogram.WithLabelValues(sl.interval.String()).Observe(
+			time.Since(last).Seconds(),
+		)
 	}
 
 	var total, added, seriesAdded, bytesRead int
@@ -1458,7 +1457,10 @@ func (sl *scrapeLoop) scrapeAndReport(last, appendTime time.Time, errc chan<- er
 			sl.l.Warn("Append failed", "err", err)
 		}
 		if errc != nil {
-			errc <- forcedErr
+			select {
+			case errc <- forcedErr:
+			case <-sl.ctx.Done():
+			}
 		}
 
 		return start
@@ -1495,7 +1497,10 @@ func (sl *scrapeLoop) scrapeAndReport(last, appendTime time.Time, errc chan<- er
 		}
 		sl.scrapeFailureLoggerMtx.RUnlock()
 		if errc != nil {
-			errc <- scrapeErr
+			select {
+			case errc <- scrapeErr:
+			case <-sl.ctx.Done():
+			}
 		}
 		if errors.Is(scrapeErr, errBodySizeLimit) {
 			bytesRead = -1
@@ -1660,7 +1665,7 @@ func (sl *scrapeLoop) append(app storage.Appender, b []byte, contentType string,
 		IgnoreNativeHistograms:                  !sl.enableNativeHistogramScraping,
 		ConvertClassicHistogramsToNHCB:          sl.convertClassicHistToNHCB,
 		KeepClassicOnClassicAndNativeHistograms: sl.alwaysScrapeClassicHist,
-		OpenMetricsSkipCTSeries:                 sl.enableCTZeroIngestion,
+		OpenMetricsSkipSTSeries:                 sl.enableSTZeroIngestion,
 		FallbackContentType:                     sl.fallbackScrapeProtocol,
 	})
 	if p == nil {
@@ -1801,21 +1806,21 @@ loop:
 		if seriesAlreadyScraped && parsedTimestamp == nil {
 			err = storage.ErrDuplicateSampleForTimestamp
 		} else {
-			if sl.enableCTZeroIngestion {
-				if ctMs := p.CreatedTimestamp(); ctMs != 0 {
+			if sl.enableSTZeroIngestion {
+				if stMs := p.StartTimestamp(); stMs != 0 {
 					if isHistogram {
 						if h != nil {
-							ref, err = app.AppendHistogramCTZeroSample(ref, lset, t, ctMs, h, nil)
+							ref, err = app.AppendHistogramSTZeroSample(ref, lset, t, stMs, h, nil)
 						} else {
-							ref, err = app.AppendHistogramCTZeroSample(ref, lset, t, ctMs, nil, fh)
+							ref, err = app.AppendHistogramSTZeroSample(ref, lset, t, stMs, nil, fh)
 						}
 					} else {
-						ref, err = app.AppendCTZeroSample(ref, lset, t, ctMs)
+						ref, err = app.AppendSTZeroSample(ref, lset, t, stMs)
 					}
-					if err != nil && !errors.Is(err, storage.ErrOutOfOrderCT) { // OOO is a common case, ignoring completely for now.
-						// CT is an experimental feature. For now, we don't need to fail the
+					if err != nil && !errors.Is(err, storage.ErrOutOfOrderST) { // OOO is a common case, ignoring completely for now.
+						// ST is an experimental feature. For now, we don't need to fail the
 						// scrape on errors updating the created timestamp, log debug.
-						sl.l.Debug("Error when appending CT in scrape loop", "series", string(met), "ct", ctMs, "t", t, "err", err)
+						sl.l.Debug("Error when appending ST in scrape loop", "series", string(met), "ct", stMs, "t", t, "err", err)
 					}
 				}
 			}
@@ -1833,7 +1838,7 @@ loop:
 
 		if err == nil {
 			if (parsedTimestamp == nil || sl.trackTimestampsStaleness) && ce != nil {
-				sl.cache.trackStaleness(ce.hash, ce)
+				sl.cache.trackStaleness(ce.ref, ce)
 			}
 		}
 
@@ -1854,7 +1859,7 @@ loop:
 			if ce != nil && (parsedTimestamp == nil || sl.trackTimestampsStaleness) {
 				// Bypass staleness logic if there is an explicit timestamp.
 				// But make sure we only do this if we have a cache entry (ce) for our series.
-				sl.cache.trackStaleness(hash, ce)
+				sl.cache.trackStaleness(ref, ce)
 			}
 			if sampleAdded && sampleLimitErr == nil && bucketLimitErr == nil {
 				seriesAdded++
@@ -1913,7 +1918,7 @@ loop:
 			if !seriesCached || lastMeta.lastIterChange == sl.cache.iter {
 				// In majority cases we can trust that the current series/histogram is matching the lastMeta and lastMFName.
 				// However, optional TYPE etc metadata and broken OM text can break this, detect those cases here.
-				// TODO(bwplotka): Consider moving this to parser as many parser users end up doing this (e.g. CT and NHCB parsing).
+				// TODO(bwplotka): Consider moving this to parser as many parser users end up doing this (e.g. ST and NHCB parsing).
 				if isSeriesPartOfFamily(lset.Get(labels.MetricName), lastMFName, lastMeta.Type) {
 					if _, merr := app.UpdateMetadata(ref, lset, lastMeta.Metadata); merr != nil {
 						// No need to fail the scrape on errors appending metadata.
