@@ -53,7 +53,7 @@ type writeHandler struct {
 	samplesWithInvalidLabelsTotal  prometheus.Counter
 	samplesAppendedWithoutMetadata prometheus.Counter
 
-	ingestCTZeroSample      bool
+	ingestSTZeroSample      bool
 	enableTypeAndUnitLabels bool
 	appendMetadata          bool
 }
@@ -65,7 +65,7 @@ const maxAheadTime = 10 * time.Minute
 //
 // NOTE(bwplotka): When accepting v2 proto and spec, partial writes are possible
 // as per https://prometheus.io/docs/specs/remote_write_spec_2_0/#partial-write.
-func NewWriteHandler(logger *slog.Logger, reg prometheus.Registerer, appendable storage.Appendable, acceptedMsgs remoteapi.MessageTypes, ingestCTZeroSample, enableTypeAndUnitLabels, appendMetadata bool) http.Handler {
+func NewWriteHandler(logger *slog.Logger, reg prometheus.Registerer, appendable storage.Appendable, acceptedMsgs remoteapi.MessageTypes, ingestSTZeroSample, enableTypeAndUnitLabels, appendMetadata bool) http.Handler {
 	h := &writeHandler{
 		logger:     logger,
 		appendable: appendable,
@@ -82,7 +82,7 @@ func NewWriteHandler(logger *slog.Logger, reg prometheus.Registerer, appendable
 			Help:      "The total number of received remote write samples (and histogram samples) which were ingested without corresponding metadata.",
 		}),
 
-		ingestCTZeroSample:      ingestCTZeroSample,
+		ingestSTZeroSample:      ingestSTZeroSample,
 		enableTypeAndUnitLabels: enableTypeAndUnitLabels,
 		appendMetadata:          appendMetadata,
 	}
@@ -329,7 +329,11 @@ func (h *writeHandler) appendV2(app storage.Appender, req *writev2.Request, rs *
 		if h.enableTypeAndUnitLabels && (m.Type != model.MetricTypeUnknown || m.Unit != "") {
 			slb := labels.NewScratchBuilder(ls.Len() + 2) // +2 for __type__ and __unit__
 			ls.Range(func(l labels.Label) {
-				slb.Add(l.Name, l.Value)
+				// Skip __type__ and __unit__ labels if they exist in the incoming labels.
+				// They will be added from metadata to avoid duplicates.
+				if l.Name != model.MetricTypeLabel && l.Name != model.MetricUnitLabel {
+					slb.Add(l.Name, l.Value)
+				}
 			})
 			schema.Metadata{Type: m.Type, Unit: m.Unit}.AddToLabels(&slb)
 			slb.Sort()
@@ -358,13 +362,13 @@ func (h *writeHandler) appendV2(app storage.Appender, req *writev2.Request, rs *
 		allSamplesSoFar := rs.AllSamples()
 		var ref storage.SeriesRef
 		for _, s := range ts.Samples {
-			if h.ingestCTZeroSample && s.StartTimestamp != 0 && s.Timestamp != 0 {
-				ref, err = app.AppendCTZeroSample(ref, ls, s.Timestamp, s.StartTimestamp)
+			if h.ingestSTZeroSample && s.StartTimestamp != 0 && s.Timestamp != 0 {
+				ref, err = app.AppendSTZeroSample(ref, ls, s.Timestamp, s.StartTimestamp)
 				// We treat OOO errors specially as it's a common scenario given:
 				// * We can't tell if ST was already ingested in a previous request.
 				// * We don't check if ST changed for stream of samples (we typically have one though),
 				// as it's checked in the AppendSTZeroSample reliably.
-				if err != nil && !errors.Is(err, storage.ErrOutOfOrderCT) {
+				if err != nil && !errors.Is(err, storage.ErrOutOfOrderST) {
 					h.logger.Debug("Error when appending ST from remote write request", "err", err, "series", ls.String(), "start_timestamp", s.StartTimestamp, "timestamp", s.Timestamp)
 				}
 			}
@@ -389,13 +393,13 @@ func (h *writeHandler) appendV2(app storage.Appender, req *writev2.Request, rs *
 
 		// Native Histograms.
 		for _, hp := range ts.Histograms {
-			if h.ingestCTZeroSample && hp.StartTimestamp != 0 && hp.Timestamp != 0 {
+			if h.ingestSTZeroSample && hp.StartTimestamp != 0 && hp.Timestamp != 0 {
 				ref, err = h.handleHistogramZeroSample(app, ref, ls, hp, hp.StartTimestamp)
 				// We treat OOO errors specially as it's a common scenario given:
 				// * We can't tell if ST was already ingested in a previous request.
 				// * We don't check if ST changed for stream of samples (we typically have one though),
 				// as it's checked in the ingestSTZeroSample reliably.
-				if err != nil && !errors.Is(err, storage.ErrOutOfOrderCT) {
+				if err != nil && !errors.Is(err, storage.ErrOutOfOrderST) {
 					h.logger.Debug("Error when appending ST from remote write request", "err", err, "series", ls.String(), "start_timestamp", hp.StartTimestamp, "timestamp", hp.Timestamp)
 				}
 			}
@@ -480,9 +484,9 @@ func (h *writeHandler) appendV2(app storage.Appender, req *writev2.Request, rs *
 func (*writeHandler) handleHistogramZeroSample(app storage.Appender, ref storage.SeriesRef, l labels.Labels, hist writev2.Histogram, st int64) (storage.SeriesRef, error) {
 	var err error
 	if hist.IsFloatHistogram() {
-		ref, err = app.AppendHistogramCTZeroSample(ref, l, hist.Timestamp, st, nil, hist.ToFloatHistogram())
+		ref, err = app.AppendHistogramSTZeroSample(ref, l, hist.Timestamp, st, nil, hist.ToFloatHistogram())
 	} else {
-		ref, err = app.AppendHistogramCTZeroSample(ref, l, hist.Timestamp, st, hist.ToIntHistogram(), nil)
+		ref, err = app.AppendHistogramSTZeroSample(ref, l, hist.Timestamp, st, hist.ToIntHistogram(), nil)
 	}
 	return ref, err
 }
@@ -499,9 +503,11 @@ type OTLPOptions struct {
 	LookbackDelta time.Duration
 	// Add type and unit labels to the metrics.
 	EnableTypeAndUnitLabels bool
-	// IngestCTZeroSample enables writing zero samples based on the start time
+	// IngestSTZeroSample enables writing zero samples based on the start time
 	// of metrics.
-	IngestCTZeroSample bool
+	IngestSTZeroSample bool
+	// AppendMetadata enables writing metadata to WAL when metadata-wal-records feature is enabled.
+	AppendMetadata bool
 }
 
 // NewOTLPWriteHandler creates a http.Handler that accepts OTLP write requests and
@@ -518,8 +524,9 @@ func NewOTLPWriteHandler(logger *slog.Logger, reg prometheus.Registerer, appenda
 		config:                  configFunc,
 		allowDeltaTemporality:   opts.NativeDelta,
 		lookbackDelta:           opts.LookbackDelta,
-		ingestCTZeroSample:      opts.IngestCTZeroSample,
+		ingestSTZeroSample:      opts.IngestSTZeroSample,
 		enableTypeAndUnitLabels: opts.EnableTypeAndUnitLabels,
+		appendMetadata:          opts.AppendMetadata,
 		// Register metrics.
 		metrics: otlptranslator.NewCombinedAppenderMetrics(reg),
 	}
@@ -560,8 +567,9 @@ type rwExporter struct {
 	config                  func() config.Config
 	allowDeltaTemporality   bool
 	lookbackDelta           time.Duration
-	ingestCTZeroSample      bool
+	ingestSTZeroSample      bool
 	enableTypeAndUnitLabels bool
+	appendMetadata          bool
 
 	// Metrics.
 	metrics otlptranslator.CombinedAppenderMetrics
@@ -573,7 +581,7 @@ func (rw *rwExporter) ConsumeMetrics(ctx context.Context, md pmetric.Metrics) er
 		Appender: rw.appendable.Appender(ctx),
 		maxTime:  timestamp.FromTime(time.Now().Add(maxAheadTime)),
 	}
-	combinedAppender := otlptranslator.NewCombinedAppender(app, rw.logger, rw.ingestCTZeroSample, rw.metrics)
+	combinedAppender := otlptranslator.NewCombinedAppender(app, rw.logger, rw.ingestSTZeroSample, rw.appendMetadata, rw.metrics)
 	converter := otlptranslator.NewPrometheusConverter(combinedAppender)
 	annots, err := converter.FromMetrics(ctx, md, otlptranslator.Settings{
 		AddMetricSuffixes:                    otlpCfg.TranslationStrategy.ShouldAddSuffixes(),
@@ -693,19 +701,23 @@ func (app *remoteWriteAppender) Append(ref storage.SeriesRef, lset labels.Labels
 }
 
 func (app *remoteWriteAppender) AppendHistogram(ref storage.SeriesRef, l labels.Labels, t int64, h *histogram.Histogram, fh *histogram.FloatHistogram) (storage.SeriesRef, error) {
+	var err error
 	if t > app.maxTime {
 		return 0, fmt.Errorf("%w: timestamp is too far in the future", storage.ErrOutOfBounds)
 	}
 
 	if h != nil && histogram.IsExponentialSchemaReserved(h.Schema) && h.Schema > histogram.ExponentialSchemaMax {
-		h = h.ReduceResolution(histogram.ExponentialSchemaMax)
+		if err = h.ReduceResolution(histogram.ExponentialSchemaMax); err != nil {
+			return 0, err
+		}
 	}
 	if fh != nil && histogram.IsExponentialSchemaReserved(fh.Schema) && fh.Schema > histogram.ExponentialSchemaMax {
-		fh = fh.ReduceResolution(histogram.ExponentialSchemaMax)
+		if err = fh.ReduceResolution(histogram.ExponentialSchemaMax); err != nil {
+			return 0, err
+		}
 	}
 
-	ref, err := app.Appender.AppendHistogram(ref, l, t, h, fh)
-	if err != nil {
+	if ref, err = app.Appender.AppendHistogram(ref, l, t, h, fh); err != nil {
 		return 0, err
 	}
 	return ref, nil
